%% Testing Matlab functions for ONLY ONE-COMPONENT damped sine
close all, clear all, clc

pkg load communications  % in Octave load the function awgn() from comm package

signal = 3;       % 1 - low freq, very fast decay = very fast exp()
                  % 2 - very low freq, very slow decay = almost sin()
                  % 3 - mid freq, mid decay = typical, both exp() and sin()
                  % 4 - mechanical spectroscopy
N = 500;          % number of samples
SNR = 120;         % signal to noise ratio

%% Test signals
K = 1;            % number of components, in this program always 1
n = 0 : N-1;      % sample indexes
if( signal == 1 ) % low frequency, fast decay
    Om = 10.2*(2*pi)/N;     % 10.2
    D = 1e-1;               % 0.1
    A = 1.3; p = -pi/2;
end
if( signal == 2 ) % very low frequency, very slow decay
    Om = 5.2*(2*pi)/N;      % 5.2
    D = 1e-4;               % 0.0001
    A = 2.3; p = -pi/4;
end
if( signal == 3 ) % typical frequency, typical decay
    Om = 20.2*(2*pi)/N;      % 20.2
    D = 1e-2;                % 0.01
    A = 2.3; p = -pi/4;
end
    
x  = A * exp(-n*D) .* cos(Om*n+p);  % damped sin    
y = awgn( x, SNR );                 % Gaussian noise, SNR in [dB]

%% Plot signal
figure;
subplot(211); plot(x); grid; title('x(n)'); 
subplot(212); plot(y); grid; title('y(n) = x(n) + noise'); pause

%% Matlab functions

L = 2*K;  % [a(0) a(1) a(2) ] = [ 1, a1, a2 ]

% Levinson
r = xcorr(y,'biased');      % biased=1/N, unbiased=1/(N-k)
r = r(N:N+L);               % THE SAME AS #1
a1 = levinson(r,L);
p = roots(a1); p = log(p); De(1) = -real(p(1)); Ome(1) = imag(p(1));

% Yule-Walker autocorrelation method
a2 = aryule(y,L);
p = roots(a2); p = log(p); De(2) = -real(p(1)); Ome(2) = imag(p(1));

% Burg
a3 = arburg(y,L);
p = roots(a3); p = log(p); De(3) = -real(p(1)); Ome(3) = imag(p(1));

C = 4; NI = 10; r=3;  % C=4, r=3 are the best
[Om4,  D4 ] = fHilbert(y,K);       De(4)  = D4;   Ome(4)  = Om4; 
[Om5,  D5 ] = fProny(y,K);         De(5)  = D5;   Ome(5)  = Om5;
[Om6,  D6 ] = fSTMB(y,K,NI);       De(6)  = D6;   Ome(6)  = Om6;
[Om7,  D7 ] = fKT(y,K);            De(7)  = D7;   Ome(7)  = Om7;
[Om8,  D8 ] = fMatPen(y,K);        De(8)  = D8;   Ome(8)  = Om8;
[Om9,  D9 ] = fYuleWalker(y,K,C);  De(9)  = D9;   Ome(9)  = Om9;
[Om10     ] = fPisarenko(y,K,C);   De(10) = Inf;  Ome(10) = Om10;
[Om11     ] = fESPRIT(y,K);        De(11) = Inf;  Ome(11) = Om11;
[Om12, D12] = fIpDFT_BY1_LC(y,NI); De(12) = D12;  Ome(12) = Om12;
[Om13, D13] = fFastTLS(y,r);       De(13) = D13;  Ome(13) = Om13;

methods = [ ...
'levinson   ';
'yule-walker';
'arburg     ';
'fHilbert   ';
'fProny     ';
'fSTMB      ';
'fKT        ';
'fMatPen    ';
'fYuleWalker';
'fPisarenko ';
'fESPRIT    ';
'fIpDFT     ';
'fFastTLS   '];

fprintf('input      : Om  = %12.8f, D  = %12.8f \n', Om, D);
[dummy, ind] = sort( abs(Ome-Om), 'ascend');
disp('BEST OMEGA ========================================');
for k = 1 : length(De)
    fprintf('%s: Ome = %12.8f, De = %12.8f \n',methods(ind(k),:),Ome(ind(k)),De(ind(k)));
end
disp('MINIMUM ERROR OMEGA [%] ===========================');
for k = 1 : length(De)
    fprintf('%s: Ome = %12.8f, De = %12.8f \n',methods(ind(k),:),100*abs(Ome(ind(k))-Om)/Om,100*abs(De(ind(k))-D)/D);
end
[dummy,ind] = sort( abs(De-D), 'ascend');
pause
disp('BEST DAMPING ======================================');
for k = 1 : length(De)
    fprintf('%s: Ome = %12.8f, De = %f10 \n',methods(ind(k),:),Ome(ind(k)),De(ind(k)));
end
disp('MINIMUM ERROR DAMPING [%] =========================');
for k = 1 : length(De)
    fprintf('%s: Ome = %12.8f, De = %12.8f \n',methods(ind(k),:),100*abs(Ome(ind(k))-Om)/Om,100*abs(De(ind(k))-D)/D);
end